// CDEstimate.c MEX file to estimate da and df values and perform variance test
#include "mex.h"
#include <math.h>
#define nsp_UsesAll	// uses Intel library functions
#include "nsp.h"
/* this is a mex function specifically designed for use with reSynth.m */
/* error trapping is kept to a minimum to enhance speed */
static double interp1DLin(double *x,double *Y,double xi,int len)
{
	// function to perform 1D linear interpolation
	int idx;

	// first constrain value
	if(xi >= x[len-1])	
		return Y[len-1];
	else if(xi <= x[0])
		return Y[0];
	else	{
		// do linear interpolation 
		idx = 0;
		while(x[idx]<xi)	// have to search for closest value as x is not evenly spaced
			idx++;
		return Y[idx-1]+(((xi-x[idx-1])/(x[idx]-x[idx-1]))*(Y[idx]-Y[idx-1]));
	}
}
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	// input and output arrays
	double *outSineIndex,*df,*da,*daX,*daY,*dfX,*dfY,*chiSquared,*chiExpect,*sineIndex,*chiSquaredThreshold,*iterations;
	// internal arrays
	double *dfEst,*daEst,**dfTable,**daTable,**chiTable,*tempSineIndex,*newSineIndex,daPointer,dfPointer,daPointerMult,dfPointerMult,dfPol,daPol,lo,hi; 
	double p11,p12,p21,p22,chiX;
	int iIterations,n,N,m,c,M,ix,dfP1,dfP2,daP1,daP2,deBug;
	
	// get pointers to input arrays
	df = mxGetPr(prhs[0]);
	da = mxGetPr(prhs[1]);
	daX = mxGetPr(prhs[2]);
	daY = mxGetPr(prhs[3]);
	dfX = mxGetPr(prhs[4]);
	dfY = mxGetPr(prhs[5]);
	chiSquared = mxGetPr(prhs[6]);
	chiExpect = mxGetPr(prhs[7]);
	sineIndex = mxGetPr(prhs[8]);
	chiSquaredThreshold = mxGetPr(prhs[9]);
	iterations = mxGetPr(prhs[10]);

	iIterations = (int)iterations[0];
	daPointerMult = 99.0/96.0;
	dfPointerMult = 99.0/260.0;
	dfEst = (double *) mxMalloc((iIterations+1)*sizeof(double));
	daEst = (double *) mxMalloc((iIterations+1)*sizeof(double));

	N = mxGetN(prhs[8]);	// N is length of sineIndex
	M = mxGetN(prhs[3]);	// M is table dimension (only need to get one since tables are square)
	newSineIndex = (double *) mxMalloc(N*sizeof(double));

	// create 2D arrays and copy data 
	dfTable = (double **) mxMalloc(M*sizeof(double *));
	daTable = (double **) mxMalloc(M*sizeof(double *));
	chiTable = (double **) mxMalloc(M*sizeof(double *));
	for (n=0;n<M;n++)	{
		dfTable[n] = (double *) mxMalloc(M*sizeof(double));
		daTable[n] = (double *) mxMalloc(M*sizeof(double));
		chiTable[n] = (double *) mxMalloc(M*sizeof(double));
	}

	for (n=0;n<M;n++)	{
		nspdbCopy(dfX+(n*M),dfTable[n],M);
		nspdbCopy(daX+(n*M),daTable[n],M);
		nspdbCopy(chiExpect+(n*M),chiTable[n],M);
	}

	c = 0;	// outSineIndex 
	// estimation process
	for (n=0;n<N;n++)	{
		ix = (int)sineIndex[n]-1;
		// find absolute value and store polarity
		if (da[ix]<0)	{
			daPol = -1.0;
			da[ix]*=daPol;
		}
		else
			daPol = 1.0;

		if (df[ix]<0)	{
			dfPol = -1.0;
			df[ix]*=dfPol;
		}
		else
			dfPol = 1.0;

		// estimate da first, assuming df is 0
		daEst[0] = interp1DLin(daTable[0],daY,da[ix],M);
		daPointer = daEst[0]*daPointerMult;
		if (daPointer<0)
			daPointer = 0;
		if (daPointer>99)
			daPointer=99;
		// then estimate df with knowledge of da
		deBug = (int)ceil(daPointer);
		deBug = (int)floor(daPointer);
		hi = interp1DLin(dfTable[(int)ceil(daPointer)],dfY,df[ix],M);
		lo = interp1DLin(dfTable[(int)floor(daPointer)],dfY,df[ix],M);
		dfEst[0] = lo + ((daPointer - floor(daPointer))*(hi-lo));
		if (dfEst[0] < 0.0)
			dfEst[0] = 0.0;
		if (dfEst[0] > 260.0)
			dfEst[0] = 260.0;

		// repeat as required
		m = 0;	// for iterations==1 case
		if (iIterations>1)	{
			for (m=1;m<iIterations;m++)	{
				// use df estimate to improve da estimate
				dfPointer = dfEst[m-1]*dfPointerMult;
				if (dfPointer<0)
					dfPointer = 0;
				if (dfPointer>99)
					dfPointer=99;
				hi = interp1DLin(daTable[(int)ceil(dfPointer)],daY,da[ix],M);
				lo = interp1DLin(daTable[(int)floor(dfPointer)],daY,da[ix],M);
				daEst[m] = lo + ((dfPointer - floor(dfPointer))*(hi-lo));
				if (daEst[m] < 0.0)
					daEst[m] = 0.0;
				if (daEst[m] > 96.0)
					daEst[m] = 96.0;
				 // use da estimate to improve df estimate
				daPointer = daEst[m]*daPointerMult;
				if (daPointer<0)
					daPointer = 0;
				if (daPointer>99)
					daPointer=99;
				hi = interp1DLin(dfTable[(int)ceil(daPointer)],dfY,df[ix],M);
				lo = interp1DLin(dfTable[(int)floor(daPointer)],dfY,df[ix],M);
				dfEst[m] = lo + ((daPointer - floor(daPointer))*(hi-lo));
				if (dfEst[m] < 0.0)
					dfEst[m] = 0.0;
				if (dfEst[m] > 260.0)
					dfEst[m] = 260.0;
			}
		}
		if (iIterations>3)	{
			// take mean of last two values of daEst and dfEst
			df[ix] = 0.5*(dfEst[m]+dfEst[m-1]);
			da[ix] = 0.5*(daEst[m]+daEst[m-1]);
		}
		else	{
			df[ix] = dfEst[m];
			da[ix] = daEst[m];
		}

		// now examine variance behaviour
		dfPointer = df[ix]*dfPointerMult;
		daPointer = da[ix]*daPointerMult;
		if (dfPointer<=0)	{
			dfPointer = 0;
			dfP1 = 0;
			dfP2 = 1;
		}
		else if (dfPointer>=(double)M-1)	{
			dfPointer = (double)M-1;
			dfP1 = M-2;
			dfP2 = M-1;
		}
		else	{
			dfP1 = (int)floor(dfPointer);
			dfP2 = (int)ceil(dfPointer);
		}
		if (daPointer<=0)	{
			daPointer = 0;
			daP1 = 0;
			daP2 = 1;
		}
		else if (daPointer>=(double)M-1)	{
			daP1 = M-2;
			daP2 = M-1;
			daPointer = (double)M-1;
		}
		else	{
			daP1 = (int)floor(daPointer);
			daP2 = (int)ceil(daPointer);
		}

		//find weighted points from coordinates
		p11 = chiTable[daP1][dfP1]*((double)daP2-daPointer)*((double)dfP2-dfPointer);
		p21 = chiTable[daP2][dfP1]*(daPointer-(double)daP1)*((double)dfP2-dfPointer);
		p12 = chiTable[daP1][dfP2]*((double)daP2-daPointer)*(dfPointer-(double)dfP1);
		p22 = chiTable[daP2][dfP2]*(daPointer-(double)daP1)*(dfPointer-(double)dfP1);
		chiX = p11+p21+p12+p22;
		chiSquared[ix] = fabs(chiSquared[ix] - chiX);
		if(chiSquared[ix]<=chiSquaredThreshold[0])	{
			// only place index in new array if it passes variance test
			newSineIndex[c] = (double)ix + 1.0;	 // + 1 because MATLAB vectors start at 1
			c++;
		}		
		// apply polarity to estimates
		da[ix] *= daPol;
		df[ix] *= dfPol;
	}
	

	// copy newSineIndex to output
	plhs[0] = mxCreateDoubleMatrix(1,c,mxREAL);
	outSineIndex = mxGetPr(plhs[0]);
	nspdbCopy(newSineIndex,outSineIndex,c);
	
}











