// CEQ.c MEX file to perform parametric equalisation
#include "mex.h"
#include <math.h>
#define nsp_UsesAll	// uses Intel library functions
#include "nsp.h"
/* this is a mex function specifically designed for use with reSynth.m */
/* error trapping is kept to a minimum to enhance speed */
void OrfanidisCoeffCalc(double *coeffs,double G0,double G,double GB,double W0,double DW)
{
	double G2,GB2,G02,G2_GB2,GB2_G02,GB2G02G2,Pi2,G2Pi2,G2_G02,W02,DW2;	// pre-calculated
	double t,b,G12,G1,W2,C,D,E,F,denom,temp;	// see Orfanidis AES paper for description
	// pre calculate some variables
	G2 = G*G;
	GB2 = GB*GB;
	G02 = G0*G0;
	G2_GB2 = G2-GB2;
	GB2_G02 = GB2-G02;
	GB2G02G2 = GB2_G02/G2_GB2;
	Pi2 = NSP_PI*NSP_PI;
	G2Pi2 = G2*Pi2;
	G2_G02 = G2-G02;
	W02 = W0*W0;
	DW2 = DW*DW;

	// do main calculation
	temp = (W0*W0)-Pi2;
	temp *= temp;
	t = (G02*((W02-Pi2)*(W02-Pi2)))+(G2*Pi2*DW2*GB2G02G2);
	b = temp + (Pi2*DW2*GB2G02G2);
	G12 = t/b;
	G1 = sqrt(G12);
	W0 = tan(W0*0.5);
	W2 = sqrt((G2-G12)/G2_G02)*W0*W0;
	DW = (1 + (sqrt(GB2_G02/fabs(GB2-G12))*W2))*tan(DW*0.5);
	C = ((DW*DW)*fabs(GB2-G12))-(2*W2*(fabs(GB2-(G0*G1))-sqrt(GB2_G02*fabs(GB2-G12))));
	D = 2*W2*(fabs(G2-(G0*G1))-sqrt(G2_G02*(G2-G12)));
	E = sqrt((C+D)/G2_GB2);
	F = sqrt((G2*C+GB2*D)/G2_GB2);  
	denom = 1/(1+W2+E);
	coeffs[0] = (G1+(G0*W2)+F)*denom;
	coeffs[1] = (-2*(G1-(G0*W2)))*denom;
	coeffs[2] = (G1+(G0*W2)-F)*denom;
	coeffs[3] = (-2*(1-W2))*denom;
	coeffs[4] = (1+W2-E)*denom;
}
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	// input and output arrays
	double *noise,*undecMag,*undecFreq,*undecBW,*num_dec,*num_undec,*xnm1,*xnm2,*ynm1,*ynm2,
				 *synLength,*Fs;

	// internal arrays
	double *temp,*dFreq,*dBW,*dMag;
	mxArray *mxA;	// used as temporary pointer to cell arrays
	double filterCoeff[5];	// EQ is bi quad filter so 5 coefficients
	double decFilter[5];	// needed as temporary storage for decimated part

	// output array
	double *noiseOut;

	// internal variables
	int iNum_dec,iNum_undec,iSynLength,iDecLen,iDecSamples,m,M,n,k,K,mxN,mxM,npnu,idxOffset,iDeBug;
	double freqMult,scaleGain,dM,dB96Mult,sqrt0_5,dDeBug,filterTemp;

	// get pointers to input arrays
	noise = mxGetPr(prhs[0]);
	undecMag = mxGetPr(prhs[4]);
	undecFreq = mxGetPr(prhs[5]);
	undecBW = mxGetPr(prhs[6]);
	num_dec = mxGetPr(prhs[7]);
	num_undec = mxGetPr(prhs[8]);
	xnm1 = mxGetPr(prhs[9]);
	xnm2 = mxGetPr(prhs[10]);
	ynm1 = mxGetPr(prhs[11]);
	ynm2 = mxGetPr(prhs[12]);
	synLength = mxGetPr(prhs[13]);
	Fs = mxGetPr(prhs[14]);

	
	iNum_dec = (int)*num_dec;
	iNum_undec = (int)*num_undec;
	iSynLength = (int)*synLength;
	freqMult = NSP_2PI/Fs[0];

	// allocate internal arrays
	temp = mxMalloc(iSynLength*sizeof(double));	// no need to initialise since it gets overwritten

	// set up and initialise output buffer
	plhs[0] = mxCreateDoubleMatrix(1,iSynLength,mxREAL);
	noiseOut = mxGetPr(plhs[0],noiseOut);
	nspdbZero(noiseOut,iSynLength);

	dB96Mult = 2.0*pow(10,96.0/20.0);   	// multiply by 2 to give normalised wavelet output magnitude
	sqrt0_5 = sqrt(0.5);

	if (iNum_undec>0)	{
		// multiply BW and frequency arrays by freqMult
		mxM = mxGetM(prhs[5]);
		mxN = mxGetN(prhs[5]);
		nspdbMpy1(freqMult,undecFreq,mxM*mxN);
		nspdbMpy1(freqMult,undecBW,mxM*mxN);
		for (n=0;n<iNum_undec;n++)	{
			scaleGain = pow(0.5,(double)n);
			for (m=0;m<iSynLength;m++)	{
				OrfanidisCoeffCalc(filterCoeff,1,dB96Mult,sqrt0_5*dB96Mult,undecFreq[(n*iSynLength)+m],undecBW[(n*iSynLength)+m]);
				filterTemp = noise[m]*scaleGain*undecMag[(n*(iSynLength+1))+m];
				filterCoeff[0] *= filterTemp;
				filterCoeff[1] *= xnm1[n];
				filterCoeff[2] *= xnm2[n];
				filterCoeff[3] *= -1.0*ynm1[n];
				filterCoeff[4] *= -1.0*ynm2[n];
				temp[m] = nspdSum(filterCoeff,5);
				xnm2[n] = xnm1[n];
				xnm1[n] = filterTemp;
				ynm2[n] = ynm1[n];
				ynm1[n] = temp[m];
			}
			nspdbAdd2(temp,noiseOut,iSynLength);
		}
	}

	if (iNum_dec>0)	{
		for (n=0;n<iNum_dec;n++)	{
			// de-reference cells
			mxA = mxGetCell(prhs[1],n);
			dMag = mxGetPr(mxA);
			iDeBug = mxGetN(mxA);
			mxA = mxGetCell(prhs[2],n);
			iDeBug = mxGetN(mxA);
			dFreq = mxGetPr(mxA);
			M = mxGetN(mxA);	// get number of values in cell
			mxA = mxGetCell(prhs[3],n);
			dBW = mxGetPr(mxA);
			K = iSynLength/M;	// number of samples per cell value
			npnu = n + iNum_undec;
			scaleGain = pow(0.5,(double)npnu);
			for (m=0;m<M;m++)	{
				// only calculate filter coefficiants when parameters are updated
				OrfanidisCoeffCalc(filterCoeff,1,dB96Mult,sqrt0_5*dB96Mult,dFreq[m]*freqMult,dBW[m]*freqMult);
				dM = dMag[m]*scaleGain;
				idxOffset = m*K;	// pointer offset
				for (k=0;k<K;k++)	{
					filterTemp = noise[idxOffset+k]*dM;
					decFilter[0] = filterCoeff[0]*filterTemp;
					decFilter[1] = filterCoeff[1]*xnm1[npnu];
					decFilter[2] = filterCoeff[2]*xnm2[npnu];
					decFilter[3] = filterCoeff[3]* -1.0*ynm1[npnu];
					decFilter[4] = filterCoeff[4]* -1.0*ynm2[npnu];
					temp[idxOffset+k] = nspdSum(decFilter,5);
					xnm2[npnu] = xnm1[npnu];
					xnm1[npnu] = filterTemp;
					ynm2[npnu] = ynm1[npnu];
					ynm1[npnu] = temp[idxOffset+k];
				}
			}
			nspdbAdd2(temp,noiseOut,iSynLength);
		}
	}
}


				

		


	

	
	