// CFindPeaks.c MEX to fit second order polynomial to time reassignment data to generate RDA measures.
#include "mex.h"
/* this is a mex function specifically designed for use with reSynth.m */
/* error trapping is kept to a minimum to enhance speed */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	double *freq,*sineIndex,*range,*df,*da,*chiSquared,*binWidth,*iStandardFFT,*rStandardFFT,*iTimeFFT,*rTimeFFT;	
	int FFTSize,n,N,lower,iRange,a,A,ix,tix,mid;
	double *x;	// cartesian arrays for parabola fitting
	double *y;
	double temp;
	double S1;
	double Sx;
	double Sx2;
	double Sx_S1;
	double LHSmatrix[3][3];
	double RHSmatrix[3];
	double xSquared,difference;
	freq = mxGetPr(prhs[0]);
	sineIndex = mxGetPr(prhs[1]);
	range = mxGetPr(prhs[2]);
	df = mxGetPr(prhs[3]);
	da = mxGetPr(prhs[4]);
	chiSquared = mxGetPr(prhs[5]);
	binWidth = mxGetPr(prhs[6]);
	iStandardFFT = mxGetPi(prhs[7]);	// put complex FFT data into two separate arrays
	rStandardFFT = mxGetPr(prhs[7]);
	iTimeFFT = mxGetPi(prhs[8]);
	rTimeFFT = mxGetPr(prhs[8]);
	FFTSize = mxGetN(prhs[7]);
	//mexPrintf("%d",FFTSize);

	N = mxGetN(prhs[1]);	// get length of sineIndex
	iRange = (int)range[0];
	//mid = iRange/2;
	A = (2*(int)range[0])+1;
	x = (double *)mxMalloc(A*sizeof(double));
	y = (double *)mxMalloc(A*sizeof(double));
	S1 = (double)A;	// S1 (sum of 1 across fitted points)
	
	for (n=0;n<N;n++)	{
		ix = (int)sineIndex[n]-1;	// MATLAB vectors start at 1, C arrays start at 0
		lower = ix-iRange;
		LHSmatrix[0][0] = S1;	// initialise gaussian matrices
		LHSmatrix[0][1] = LHSmatrix[0][2] = LHSmatrix[1][0] = LHSmatrix[1][1] = 0;
		LHSmatrix[1][2] = LHSmatrix[2][0] = LHSmatrix[2][1] = LHSmatrix[2][2] = 0;
		RHSmatrix[0] = RHSmatrix[1] = RHSmatrix[2] = Sx = Sx2 = 0;
		for (a=0;a<A;a++)	{
			tix = lower+a;
			x[a] = (*binWidth * (double)tix)-freq[ix];
			y[a] = (iStandardFFT[tix]*iTimeFFT[tix])+(rStandardFFT[tix]*rTimeFFT[tix]);	// only the real part of complex division is needed
			y[a] /= (iStandardFFT[tix]*iStandardFFT[tix])+(rStandardFFT[tix]*rStandardFFT[tix]);
			//do summations
			LHSmatrix[0][1] = LHSmatrix[1][0] = Sx += x[a];
			LHSmatrix[0][2] = LHSmatrix[1][1] = LHSmatrix[2][0] = Sx2 += x[a] * x[a];
			LHSmatrix[1][2] = LHSmatrix[2][1] += x[a] * x[a] * x[a];
			LHSmatrix[2][2] += x[a] * x[a] * x[a] * x[a];
			RHSmatrix[0] += y[a];
			RHSmatrix[1] += x[a] * y[a];
			RHSmatrix[2] += x[a] * x[a] * y[a];
		}
		//GaussianElimination;
		//subtract (sum of x divided by A-2) multiplied by row 1 from row 2, A-2 since only 3 points are taken for fit
		Sx_S1 = Sx/S1;
		for (a = 0; a < 3; a++)	{
			temp = LHSmatrix[0][a] * Sx_S1;
			LHSmatrix[1][a] -= temp;
		}
		temp = RHSmatrix[0] * Sx_S1;
		RHSmatrix[1] -= temp;

		//divide top row by S1
		for (a = 0; a < 3; a++)	LHSmatrix[0][a] /= S1;
		RHSmatrix[0] /= S1;

		//subtract sum of x^2 multiplied by row 1 from row 3
		for (a = 0; a < 3; a++)	{
			temp = LHSmatrix[0][a] * Sx2;
			LHSmatrix[2][a] -= temp;
		}
		temp = RHSmatrix[0] * Sx2;
		RHSmatrix[2] -= temp;

		//divide second row by 2nd element of second row
		temp = LHSmatrix[1][1];
		for (a = 0; a < 3; a++)	LHSmatrix[1][a]/= temp;
		RHSmatrix[1] /= temp;

		//multiply row 2 by middle element of row 3 and subtract from row 3
		temp = LHSmatrix[2][1];
		for (a = 0; a < 3; a++)	LHSmatrix[2][a] -= LHSmatrix[1][a] * temp;
		RHSmatrix[2] -= RHSmatrix[1] * temp;

		//divide row 3 by last element of row 3
		RHSmatrix[2] /= LHSmatrix[2][2];
		LHSmatrix[2][2] = 1.0;

		//now calculate coefficients
		temp = RHSmatrix[2];
		df[ix] = RHSmatrix[1] - (temp * LHSmatrix[1][2]);
		da[ix] = RHSmatrix[0] - (df[ix] * LHSmatrix[0][1]) - (temp * LHSmatrix[0][2]);
	
		//calculate chi squared
		chiSquared[ix] = 0.0;
		for (a = 0; a < A; a++)	{
			xSquared = x[a] * x[a];
			difference = y[a] - (temp * xSquared) - (df[ix] * x[a]) - da[ix];
			chiSquared[ix] += (difference * difference);
		}
	}	
}